@@ -12,7 +12,6 @@ from .basic_types import (
     CombineReducerRegisterAction,
     CombineReducerUnregisterAction,
     CompleteReducerResult,
-    CreateStoreOptions,
     Dispatch,
     DispatchParameters,
     EventSubscriber,
@@ -23,6 +22,7 @@ from .basic_types import (
     ReducerResult,
     ReducerType,
     Scheduler,
+    StoreOptions,
     ViewDecorator,
     ViewOptions,
     ViewReturnType,
@@ -44,7 +44,6 @@ __all__ = (
     'CombineReducerRegisterAction',
     'CombineReducerUnregisterAction',
     'CompleteReducerResult',
-    'CreateStoreOptions',
     'Dispatch',
     'DispatchParameters',
     'EventSubscriber',
@@ -56,6 +55,7 @@ __all__ = (
     'ReducerType',
     'Scheduler',
     'Store',
+    'StoreOptions',
     'ViewDecorator',
     'ViewOptions',
     'ViewReturnType',
