@@ -3,28 +3,29 @@
 from __future__ import annotations
 
 import asyncio
-import functools
 import inspect
 import weakref
-from asyncio import Future, Task, iscoroutine, iscoroutinefunction
+from asyncio import iscoroutine, iscoroutinefunction
 from typing import (
     TYPE_CHECKING,
     Any,
     Concatenate,
     Generic,
-    TypeVar,
+    Literal,
     cast,
 )
 
 from redux.basic_types import (
     Action,
     Args,
-    AutorunOptions,
+    AutoAwait,
+    AutorunOptionsType,
     ComparatorOutput,
     Event,
     ReturnType,
     SelectorOutput,
     State,
+    T,
 )
 
 if TYPE_CHECKING:
@@ -33,26 +34,37 @@ if TYPE_CHECKING:
     from redux.main import Store
 
 
-T = TypeVar('T')
-
-
 class AwaitableWrapper(Generic[T]):
     """A wrapper for a coroutine to track if it has been awaited."""
 
+    _unawaited = object()
+    value: tuple[Literal[False], None] | tuple[Literal[True], T]
+
     def __init__(self, coro: Coroutine[None, None, T]) -> None:
         """Initialize the AwaitableWrapper with a coroutine."""
         self.coro = coro
-        self.awaited = False
+        self.value = (False, None)
 
     def __await__(self) -> Generator[None, None, T]:
         """Await the coroutine and set the awaited flag to True."""
-        self.awaited = True
-        return self.coro.__await__()
+        return self._wrap().__await__()
+
+    async def _wrap(self) -> T:
+        """Wrap the coroutine and set the awaited flag to True."""
+        if self.value[0] is True:
+            return self.value[1]
+        self.value = (True, await self.coro)
+        return self.value[1]
 
     def close(self) -> None:
         """Close the coroutine if it has not been awaited."""
         self.coro.close()
 
+    @property
+    def awaited(self) -> bool:
+        """Check if the coroutine has been awaited."""
+        return self.value[0] is True
+
     def __repr__(self) -> str:
         """Return a string representation of the AwaitableWrapper."""
         return f'AwaitableWrapper({self.coro}, awaited={self.awaited})'
@@ -71,7 +83,7 @@ class Autorun(
 ):
     """Run a wrapped function in response to specific state changes in the store."""
 
-    def __init__(  # noqa: C901, PLR0912
+    def __init__(  # noqa: C901, PLR0912, PLR0915
         self: Autorun,
         *,
         store: Store[State, Action, Event],
@@ -81,7 +93,7 @@ class Autorun(
             Concatenate[SelectorOutput, Args],
             ReturnType,
         ],
-        options: AutorunOptions[ReturnType],
+        options: AutorunOptionsType[ReturnType, AutoAwait],
     ) -> None:
         """Initialize the Autorun instance."""
         if hasattr(func, '__name__'):
@@ -121,7 +133,7 @@ class Autorun(
             self._func = weakref.ref(func, self.unsubscribe)
         self._is_coroutine = (
             asyncio.coroutines._is_coroutine  # pyright: ignore [reportAttributeAccessIssue]  # noqa: SLF001
-            if asyncio.iscoroutinefunction(func)
+            if asyncio.iscoroutinefunction(func) and options.auto_await is False
             else None
         )
         self._options = options
@@ -132,29 +144,39 @@ class Autorun(
             object(),
         )
         if iscoroutinefunction(func):
-            self._latest_value = Future()
-            self._latest_value.set_result(options.default_value)
+
+            async def default_value_wrapper() -> ReturnType | None:
+                return options.default_value
+
+            create_task = self._store._create_task  # noqa: SLF001
+            default_value = default_value_wrapper()
+
+            if create_task:
+                create_task(default_value)
+            self._latest_value: ReturnType = default_value
         else:
             self._latest_value: ReturnType = options.default_value
         self._subscriptions: set[
             Callable[[ReturnType], Any] | weakref.ref[Callable[[ReturnType], Any]]
         ] = set()
 
-        if self._check(store._state) and self._options.initial_call:  # noqa: SLF001
-            self._call()
+        if self.check(store._state) and self._options.initial_call:  # noqa: SLF001
+            self._should_be_called = False
+            self.call()
 
         if self._options.reactive:
-            self._unsubscribe = store._subscribe(self._react)  # noqa: SLF001
+            self._unsubscribe = store._subscribe(self.react)  # noqa: SLF001
         else:
             self._unsubscribe = None
 
-    def _react(
+    def react(
         self: Autorun,
         state: State,
     ) -> None:
         """React to state changes in the store."""
-        if self._options.reactive and self._check(state):
-            self._call()
+        if self._options.reactive and self.check(state):
+            self._should_be_called = False
+            self.call()
 
     def unsubscribe(
         self: Autorun[
@@ -195,28 +217,7 @@ class Autorun(
                 subscriber = subscriber_
             subscriber(self._latest_value)
 
-    def _task_callback(
-        self: Autorun[
-            State,
-            Action,
-            Event,
-            SelectorOutput,
-            ComparatorOutput,
-            Args,
-            ReturnType,
-        ],
-        task: Task,
-        *,
-        future: Future,
-    ) -> None:
-        task.add_done_callback(
-            lambda result: (
-                future.set_result(result.result()),
-                self.inform_subscribers(),
-            ),
-        )
-
-    def _check(
+    def check(
         self: Autorun[
             State,
             Action,
@@ -228,6 +229,7 @@ class Autorun(
         ],
         state: State | None,
     ) -> bool:
+        """Check if the autorun should be called based on the current state."""
         if state is None:
             return False
         try:
@@ -248,7 +250,7 @@ class Autorun(
         self._last_comparator_result = comparator_result
         return self._should_be_called
 
-    def _call(
+    def call(
         self: Autorun[
             State,
             Action,
@@ -261,7 +263,7 @@ class Autorun(
         *args: Args.args,
         **kwargs: Args.kwargs,
     ) -> None:
-        self._should_be_called = False
+        """Call the wrapped function with the current state of the store."""
         func = self._func() if isinstance(self._func, weakref.ref) else self._func
         if func and self._last_selector_result is not None:
             value: ReturnType = func(
@@ -270,31 +272,23 @@ class Autorun(
                 **kwargs,
             )
             create_task = self._store._create_task  # noqa: SLF001
+            previous_value = self._latest_value
             if iscoroutine(value) and create_task:
-                if self._options.auto_await:
-                    future = Future()
-                    self._latest_value = cast('ReturnType', future)
-                    create_task(
-                        value,
-                        callback=functools.partial(
-                            self._task_callback,
-                            future=future,
-                        ),
-                    )
-                else:
+                if self._options.auto_await is False:
                     if (
                         self._latest_value is not None
                         and isinstance(self._latest_value, AwaitableWrapper)
                         and not self._latest_value.awaited
                     ):
                         self._latest_value.close()
-                    self._latest_value = cast(
-                        'ReturnType',
-                        AwaitableWrapper(value),
-                    )
+                    self._latest_value = cast('ReturnType', AwaitableWrapper(value))
+                else:
+                    self._latest_value = cast('ReturnType', None)
+                    create_task(value)
             else:
                 self._latest_value = value
-            self.inform_subscribers()
+            if self._latest_value is not previous_value:
+                self.inform_subscribers()
 
     def __call__(
         self: Autorun[
@@ -311,9 +305,10 @@ class Autorun(
     ) -> ReturnType:
         """Call the wrapped function with the current state of the store."""
         state = self._store._state  # noqa: SLF001
-        self._check(state)
+        self.check(state)
         if self._should_be_called or args or kwargs or not self._options.memoization:
-            self._call(*args, **kwargs)
+            self._should_be_called = False
+            self.call(*args, **kwargs)
         return cast('ReturnType', self._latest_value)
 
     def __repr__(
