@@ -7,6 +7,8 @@ import inspect
 import queue
 import weakref
 from collections import defaultdict
+from collections.abc import Awaitable, Iterable, Sequence
+from functools import wraps
 from threading import Lock, Thread
 from typing import (
     TYPE_CHECKING,
@@ -17,21 +19,19 @@ from typing import (
     overload,
 )
 
-from redux.autorun import Autorun
 from redux.basic_types import (
     Action,
     ActionMiddleware,
     Args,
+    AutoAwait,
     AutorunDecorator,
     AutorunOptions,
-    AutorunOptionsWithDefault,
-    AutorunOptionsWithoutDefault,
+    AutorunOptionsType,
     AutorunReturnType,
     AwaitableOrNot,
     BaseAction,
     BaseEvent,
     ComparatorOutput,
-    CreateStoreOptions,
     DispatchParameters,
     Event,
     EventHandler,
@@ -44,37 +44,32 @@ from redux.basic_types import (
     SelectorOutput,
     SnapshotAtom,
     State,
+    StoreOptions,
     StrictEvent,
     SubscribeEventCleanup,
-    UnknownAutorunDecorator,
-    UnknownViewDecorator,
     ViewDecorator,
     ViewOptions,
-    ViewOptionsWithDefault,
-    ViewOptionsWithoutDefault,
     ViewReturnType,
     WithStateDecorator,
     is_complete_reducer_result,
     is_state_reducer_result,
 )
 from redux.serialization_mixin import SerializationMixin
-from redux.side_effect_runner import SideEffectRunnerThread
-from redux.with_state import WithState
 
 if TYPE_CHECKING:
-    from collections.abc import Awaitable, Callable
+    from collections.abc import Callable
 
 
 class Store(Generic[State, Action, Event], SerializationMixin):
     """Redux store for managing state and side effects."""
 
     def __init__(
-        self: Store,
-        reducer: ReducerType[State, Action, Event],
-        options: CreateStoreOptions[Action, Event] | None = None,
+        self,
+        reducer: ReducerType[State, Action | InitAction, Event | None],
+        options: StoreOptions[Action, Event] | None = None,
     ) -> None:
         """Create a new store."""
-        self.store_options = options or CreateStoreOptions()
+        self.store_options = options or StoreOptions()
         self.reducer = reducer
         self._create_task = self.store_options.task_creator
 
@@ -83,25 +78,26 @@ class Store(Generic[State, Action, Event], SerializationMixin):
 
         self._state: State | None = None
         self._listeners: set[
-            Callable[[State], Any] | weakref.ref[Callable[[State], Any]]
+            Callable[[State], AwaitableOrNot[None]]
+            | weakref.ref[Callable[[State], AwaitableOrNot[None]]]
         ] = set()
         self._event_handlers: defaultdict[
-            type[Event],
+            type[Event | FinishEvent],
             set[EventHandler | weakref.ref[EventHandler]],
         ] = defaultdict(set)
 
-        self._actions: list[Action] = []
-        self._events: list[Event] = []
+        self._actions: list[Action | InitAction] = []
+        self._events: list[Event | FinishEvent] = []
 
         self._event_handlers_queue = queue.Queue[
-            tuple[EventHandler[Event], Event] | None
+            tuple[EventHandler[Event | FinishEvent], Event | FinishEvent] | None
         ]()
         self._workers = [
-            SideEffectRunnerThread(
+            self.store_options.side_effect_runner_class(
                 task_queue=self._event_handlers_queue,
                 create_task=self._create_task,
             )
-            for _ in range(self.store_options.threads)
+            for _ in range(self.store_options.side_effect_threads)
         ]
         for worker in self._workers:
             worker.start()
@@ -111,11 +107,11 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         if self.store_options.auto_init:
             if self.store_options.scheduler:
                 self.store_options.scheduler(
-                    lambda: self.dispatch(cast('Action', InitAction())),
+                    lambda: self.dispatch(InitAction()),
                     interval=False,
                 )
             else:
-                self.dispatch(cast('Action', InitAction()))
+                self.dispatch(InitAction())
 
         if self.store_options.scheduler:
             self.store_options.scheduler(self.run, interval=True)
@@ -145,7 +141,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                     self._call_listeners(self._state)
 
                 if isinstance(action, FinishAction):
-                    self._dispatch([cast('Event', FinishEvent())])
+                    self._dispatch([FinishEvent()])
 
     def _run_event_handlers(self: Store[State, Action, Event]) -> None:
         while len(self._events) > 0:
@@ -169,7 +165,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
     def clean_up(self: Store[State, Action, Event]) -> None:
         """Clean up the store."""
         self.wait_for_event_handlers()
-        for _ in range(self.store_options.threads):
+        for _ in range(self.store_options.side_effect_threads):
             self._event_handlers_queue.put_nowait(None)
         self.wait_for_event_handlers()
         for worker in self._workers:
@@ -205,17 +201,17 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         actions = [
             action
             for actions in parameters
-            for action in (actions if isinstance(actions, list) else [actions])
+            for action in (actions if isinstance(actions, Iterable) else [actions])
         ]
         self._dispatch(actions)
 
     def _dispatch(
         self: Store[State, Action, Event],
-        items: list[Action | Event],
+        items: Sequence[Action | Event | InitAction | FinishEvent | None],
     ) -> None:
         for item in items:
             if isinstance(item, BaseAction):
-                action = cast('Action', item)
+                action = item
                 for action_middleware in self._action_middlewares:
                     action_ = action_middleware(action)
                     if action_ is None:
@@ -224,7 +220,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 else:
                     self._actions.append(action)
             if isinstance(item, BaseEvent):
-                event = cast('Event', item)
+                event = item
                 for event_middleware in self._event_middlewares:
                     event_ = event_middleware(event)
                     if event_ is None:
@@ -273,10 +269,10 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         else:
             handler_ref = weakref.ref(handler)
 
-        self._event_handlers[cast('Any', event_type)].add(handler_ref)
+        self._event_handlers[cast('Event', event_type)].add(handler_ref)
 
         def unsubscribe() -> None:
-            self._event_handlers[cast('Any', event_type)].discard(handler_ref)
+            self._event_handlers[cast('Event', event_type)].discard(handler_ref)
 
         return SubscribeEventCleanup(unsubscribe=unsubscribe, handler=handler)
 
@@ -299,55 +295,22 @@ class Store(Generic[State, Action, Event], SerializationMixin):
     def _handle_finish_event(self: Store[State, Action, Event]) -> None:
         Thread(target=self._wait_for_store_to_finish).start()
 
-    @overload
     def autorun(
         self: Store[State, Action, Event],
         selector: Callable[[State], SelectorOutput],
         comparator: Callable[[State], ComparatorOutput] | None = None,
         *,
-        options: AutorunOptionsWithoutDefault | None = None,
-    ) -> UnknownAutorunDecorator[SelectorOutput]: ...
-    @overload
-    def autorun(
-        self: Store[State, Action, Event],
-        selector: Callable[[State], SelectorOutput],
-        comparator: Callable[[State], ComparatorOutput] | None = None,
-        *,
-        options: AutorunOptionsWithDefault[ReturnType],
-    ) -> AutorunDecorator[SelectorOutput, ReturnType]: ...
-    def autorun(
-        self: Store[State, Action, Event],
-        selector: Callable[[State], SelectorOutput],
-        comparator: Callable[[State], ComparatorOutput] | None = None,
-        *,
-        options: AutorunOptions[ReturnType] | None = None,
-    ) -> (
-        AutorunDecorator[SelectorOutput, ReturnType]
-        | UnknownAutorunDecorator[SelectorOutput]
-    ):
+        options: AutorunOptionsType[ReturnType, AutoAwait] | None = None,
+    ) -> AutorunDecorator[ReturnType, SelectorOutput, AutoAwait]:
         """Create a new autorun, reflecting on state changes."""
 
-        @overload
-        def autorun_decorator(
-            func: Callable[
-                Concatenate[SelectorOutput, Args],
-                ReturnType,
-            ],
-        ) -> AutorunReturnType[ReturnType, Args]: ...
-        @overload
-        def autorun_decorator(
-            func: Callable[
-                Concatenate[SelectorOutput, Args],
-                Awaitable[ReturnType],
-            ],
-        ) -> AutorunReturnType[Awaitable[ReturnType], Args]: ...
         def autorun_decorator(
             func: Callable[
                 Concatenate[SelectorOutput, Args],
                 AwaitableOrNot[ReturnType],
             ],
-        ) -> AutorunReturnType[AwaitableOrNot[ReturnType], Args]:
-            return Autorun(
+        ) -> AutorunReturnType[ReturnType, Args]:
+            return self.store_options.autorun_class(
                 store=self,
                 selector=selector,
                 comparator=comparator,
@@ -355,30 +318,14 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 options=options or AutorunOptions(),
             )
 
-        return autorun_decorator
+        return cast('AutorunDecorator', autorun_decorator)
 
-    @overload
-    def view(
-        self: Store[State, Action, Event],
-        selector: Callable[[State], SelectorOutput],
-        *,
-        options: ViewOptionsWithoutDefault | None = None,
-    ) -> UnknownViewDecorator[SelectorOutput]: ...
-    @overload
-    def view(
-        self: Store[State, Action, Event],
-        selector: Callable[[State], SelectorOutput],
-        *,
-        options: ViewOptionsWithDefault[ReturnType],
-    ) -> ViewDecorator[SelectorOutput, ReturnType]: ...
     def view(
         self: Store[State, Action, Event],
         selector: Callable[[State], SelectorOutput],
         *,
         options: ViewOptions[ReturnType] | None = None,
-    ) -> (
-        ViewDecorator[SelectorOutput, ReturnType] | UnknownViewDecorator[SelectorOutput]
-    ):
+    ) -> ViewDecorator[ReturnType, SelectorOutput]:
         """Create a new view, throttling calls for unchanged selector results."""
 
         @overload
@@ -403,14 +350,14 @@ class Store(Generic[State, Action, Event], SerializationMixin):
             ],
         ) -> ViewReturnType[AwaitableOrNot[ReturnType], Args]:
             _options = options or ViewOptions()
-            return Autorun(
+            return self.store_options.autorun_class(
                 store=self,
                 selector=selector,
                 comparator=None,
                 func=cast('Callable', func),
                 options=AutorunOptions(
                     default_value=_options.default_value,
-                    auto_await=True,
+                    auto_await=False,
                     initial_call=False,
                     reactive=False,
                     memoization=_options.memoization,
@@ -442,7 +389,26 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 ReturnType,
             ],
         ) -> Callable[Args, ReturnType]:
-            return WithState(store=self, selector=selector, func=cast('Callable', func))
+            def wrapper(*args: Args.args, **kwargs: Args.kwargs) -> ReturnType:
+                if self._state is None:
+                    msg = 'Store has not been initialized yet.'
+                    raise RuntimeError(msg)
+                return func(selector(self._state), *args, **kwargs)
+
+            signature = inspect.signature(func)
+            parameters = list(signature.parameters.values())
+            if parameters and parameters[0].kind in [
+                inspect.Parameter.POSITIONAL_ONLY,
+                inspect.Parameter.POSITIONAL_OR_KEYWORD,
+            ]:
+                parameters = parameters[1:]
+
+            wrapped = wraps(func)(wrapper)
+            wrapped.__signature__ = signature.replace(  # pyright: ignore [reportAttributeAccessIssue]
+                parameters=parameters,
+            )
+
+            return wrapped
 
         return with_state_decorator
 
